@@ -78,11 +78,17 @@ class SOAP_Server extends SOAP_Base
 
     var $service = ''; //soapaction header
     var $method_namespace = null;
-    var $__options = array('use' => 'encoded',
-                           'style' => 'rpc',
-                           'parameters' => 0,
-                           'http_status_success' => '200 OK',
-                           'http_status_fault' => '500 SOAP Fault');
+
+    /**
+     * Options.
+     *
+     * @var array
+     */
+    var $_options = array('use' => 'encoded',
+                          'style' => 'rpc',
+                          'parameters' => 0,
+                          'http_status_success' => '200 OK',
+                          'http_status_fault' => '500 SOAP Fault');
 
     function SOAP_Server($options = null)
     {
@@ -91,18 +97,18 @@ class SOAP_Server extends SOAP_Base
 
         if (is_array($options)) {
             if (isset($options['use'])) {
-                $this->__options['use'] = $options['use'];
+                $this->_options['use'] = $options['use'];
             }
             if (isset($options['style'])) {
-                $this->__options['style'] = $options['style'];
+                $this->_options['style'] = $options['style'];
             }
             if (isset($options['parameters'])) {
-                $this->__options['parameters'] = $options['parameters'];
+                $this->_options['parameters'] = $options['parameters'];
             }
         }
         // assume we encode with section 5
         $this->_section5 = true;
-        if ($this->__options['use']=='literal') {
+        if ($this->_options['use']=='literal') {
             $this->_section5 = false;
         }
     }
@@ -120,9 +126,9 @@ class SOAP_Server extends SOAP_Base
     {
         /* The error handler should ignore '0' errors, eg. hidden by @ - see
          * the set_error_handler manual page. (thanks to Alan Knowles). */
-        if (!$errno || $errno == E_NOTICE ||
+        if (!$errno || !error_reporting() || $errno == E_NOTICE ||
             (defined('E_STRICT') && $errno == constant('E_STRICT'))) {
-            return;
+            return false;
         }
 
         $this->fault =& new SOAP_Fault($errmsg, 'Server', 'PHP', "Errno: $errno\nFilename: $filename\nLineno: $linenum\n");
@@ -149,9 +155,16 @@ class SOAP_Server extends SOAP_Base
 
 
     /**
-     * Parses request and posts response.
+     * Parses the request and posts or returns the response.
+     *
+     * @param string $data      The SOAP request data.
+     * @param string $endpoint  The service endpoint. Determined automatically
+     *                          if left empty.
+     * @param boolean $test
+     * @param boolean $return   Whether to return the SOAP response data
+     *                          instead of sending it to the client.
      */
-    function service($data, $endpoint = '', $test = false)
+    function service($data, $endpoint = '', $test = false, $return = false)
     {
         $response = null;
         $attachments = array();
@@ -210,7 +223,7 @@ class SOAP_Server extends SOAP_Base
             /* Handle Mime or DIME encoding. */
             /* TODO: DIME decoding should move to the transport, do it here
              * for now and for ease of getting it done. */
-            if (count($this->__attachments)) {
+            if (count($this->_attachments)) {
                 if ($useEncoding == 'Mime') {
                     $soap_msg = $this->_makeMimeMessage($soap_msg);
                 } else {
@@ -233,6 +246,13 @@ class SOAP_Server extends SOAP_Base
             }
         }
 
+        if ($return) {
+            if ($this->fault) {
+                $response = $this->fault->message();
+            }
+            return $response;
+        }
+
         $this->_sendResponse($response);
     }
 
@@ -256,10 +276,10 @@ class SOAP_Server extends SOAP_Base
         }
 
         if ($this->fault) {
-            $hdrs = $hdrs_type . ' ' . $this->__options['http_status_fault'] . "\r\n";
-            $response = $this->fault->message();
+            $hdrs = $hdrs_type . ' ' . $this->_options['http_status_fault'] . "\r\n";
+            $response = $this->fault->message($this->response_encoding);
         } else {
-            $hdrs = $hdrs_type . ' ' . $this->__options['http_status_success'] . "\r\n";
+            $hdrs = $hdrs_type . ' ' . $this->_options['http_status_success'] . "\r\n";
         }
         header($hdrs);
 
@@ -291,16 +311,16 @@ class SOAP_Server extends SOAP_Base
         if ($args) {
             /* Call method with parameters. */
             if (isset($this->soapobject) && is_object($this->soapobject)) {
-                $ret = @call_user_func_array(array(&$this->soapobject, $methodname), $args);
+                $ret = call_user_func_array(array(&$this->soapobject, $methodname), $args);
             } else {
-                $ret = @call_user_func_array($methodname, $args);
+                $ret = call_user_func_array($methodname, $args);
             }
         } else {
             /* Call method withour parameters. */
             if (is_object($this->soapobject)) {
-                $ret = @call_user_func(array(&$this->soapobject, $methodname));
+                $ret = call_user_func(array(&$this->soapobject, $methodname));
             } else {
-                $ret = @call_user_func($methodname);
+                $ret = call_user_func($methodname);
             }
         }
 
@@ -401,7 +421,7 @@ class SOAP_Server extends SOAP_Base
                         /* If there are parameters to pass. */
                         $hr =& $this->callMethod($header_method, $header_data);
                         if (PEAR::isError($hr)) {
-                            $this->_raiseSoapDefault($hr);
+                            $this->_raiseSoapFault($hr);
                             return null;
                         }
                         $results = $this->buildResult($hr, $this->return_type, $header_method, $header_val->namespace);
@@ -434,9 +454,9 @@ class SOAP_Server extends SOAP_Base
                 $this->_raiseSoapFault($opData);
                 return null;
             }
-            $this->__options['style'] = $opData['style'];
-            $this->__options['use'] = $opData['output']['use'];
-            $this->__options['parameters'] = $opData['parameters'];
+            $this->_options['style'] = $opData['style'];
+            $this->_options['use'] = $opData['output']['use'];
+            $this->_options['parameters'] = $opData['parameters'];
         }
 
         /* Does method exist? */
@@ -474,9 +494,9 @@ class SOAP_Server extends SOAP_Base
             return null;
         }
 
-        if ($this->__options['parameters'] ||
+        if ($this->_options['parameters'] ||
             !$method_response ||
-            $this->__options['style']=='rpc') {
+            $this->_options['style']=='rpc') {
             /* Get the method result. */
             if (is_null($method_response)) {
                 $return_val = null;
@@ -489,7 +509,7 @@ class SOAP_Server extends SOAP_Base
         } else {
             $methodValue =& $method_response;
         }
-        return $this->_makeEnvelope($methodValue, $header_results, $this->response_encoding);
+        return $this->makeEnvelope($methodValue, $header_results, $this->response_encoding);
     }
 
     function &__decodeRequest($request, $shift = false)
@@ -518,7 +538,7 @@ class SOAP_Server extends SOAP_Base
         if (is_object($requestArray) &&
             get_class($requestArray) == 'stdClass') {
             $requestArray = get_object_vars($requestArray);
-        } elseif ($this->__options['style'] == 'document') {
+        } elseif ($this->_options['style'] == 'document') {
             $requestArray = array($requestArray);
         }
         if (is_array($requestArray)) {
@@ -734,7 +754,7 @@ class SOAP_Server extends SOAP_Base
      */
     function addToMap($methodname, $in, $out, $namespace = null, $alias = null)
     {
-        if (!function_exists($methodname)) {
+        if (!$this->callHandler && !function_exists($methodname)) {
             $this->_raiseSoapFault('Error mapping function', '', '', 'Server');
             return false;
         }
