@@ -4,9 +4,9 @@
  *
  * PHP versions 4 and 5
  *
- * LICENSE: This source file is subject to version 3.01 of the PHP license,
+ * LICENSE: This source file is subject to version 2.02 of the PHP license,
  * that is bundled with this package in the file LICENSE, and is available at
- * through the world-wide-web at http://www.php.net/license/3_01.txt.  If you
+ * through the world-wide-web at http://www.php.net/license/2_02.txt.  If you
  * did not receive a copy of the PHP license and are unable to obtain it
  * through the world-wide-web, please send a note to license@php.net so we can
  * mail you a copy immediately.
@@ -18,7 +18,7 @@
  * @author     Chuck Hagenbuch <chuck@horde.org>   Maintenance
  * @author     Jan Schneider <jan@horde.org>       Maintenance
  * @copyright  2003-2005 The PHP Group
- * @license    http://www.php.net/license/3_01.txt  PHP License 3.01
+ * @license    http://www.php.net/license/2_02.txt  PHP License 2.02
  * @link       http://pear.php.net/package/SOAP
  */
 
@@ -27,7 +27,6 @@ require_once 'SOAP/Fault.php';
 require_once 'HTTP/Request.php';
 
 define('WSDL_CACHE_MAX_AGE', 43200);
-define('WSDL_CACHE_USE',     0); // set to zero to turn off caching
 
 /**
  * This class parses WSDL files, and can be used by SOAP::Client to properly
@@ -61,31 +60,55 @@ class SOAP_WSDL extends SOAP_Base
     var $imports = array();
     var $services = array();
     var $service = '';
-    var $uri = '';
-    var $docs = false;
 
     /**
-     * Proxy parameters
+     * URL to WSDL file.
      *
-     * @var array
+     * @var string
      */
-    var $proxy = null;
-
-    var $trace = 0;
+    var $uri;
 
     /**
-     * Use WSDL cache.
+     * Parse documentation in the WSDL?
      *
      * @var boolean
      */
-    var $cacheUse = null;
+    var $docs;
 
     /**
-     * Cache max lifetime (in seconds).
+     * Proxy parameters.
+     *
+     * @var array
+     */
+    var $proxy;
+
+    /**
+     * Enable tracing in the generated proxy class?
+     *
+     * @var boolean
+     */
+    var $trace = false;
+
+    /**
+     * Use WSDL cache?
+     *
+     * @var boolean
+     */
+    var $cacheUse;
+
+    /**
+     * WSDL cache directory.
+     *
+     * @var string
+     */
+    var $cacheDir;
+
+    /**
+     * Cache maximum lifetime (in seconds).
      *
      * @var integer
      */
-    var $cacheMaxAge = null;
+    var $cacheMaxAge;
 
     /**
      * Class to use for WSDL parsing. Can be overridden for special cases,
@@ -95,31 +118,64 @@ class SOAP_WSDL extends SOAP_Base
      */
     var $wsdlParserClass = 'SOAP_WSDL_Parser';
 
+    /**
+     * Reserved PHP keywords.
+     *
+     * @link http://www.php.net/manual/en/reserved.php
+     *
+     * @var array
+     */
+    var $_reserved = array('abstract', 'and', 'array', 'as', 'break', 'case',
+                           'catch', 'cfunction', 'class', 'clone', 'const',
+                           'continue', 'declare', 'default', 'die', 'do',
+                           'echo', 'else', 'elseif', 'empty', 'enddeclare',
+                           'endfor', 'endforeach', 'endif', 'endswitch',
+                           'endwhile', 'eval', 'exception', 'exit', 'extends',
+                           'final', 'for', 'foreach', 'function', 'global',
+                           'if', 'implements', 'include', 'include_once',
+                           'interface', 'isset', 'list', 'new', 'old_function',
+                           'or', 'php_user_filter', 'print', 'private',
+                           'protected', 'public', 'require', 'require_once',
+                           'return', 'static', 'switch', 'this', 'throw',
+                           'try', 'unset', 'use', 'var', 'while', 'xor');
+
+    /**
+     * Regular expressions for invalid PHP labels.
+     *
+     * @link http://www.php.net/manual/en/language.variables.php.
+     *
+     * @var string
+     */
+    var $_invalid = array('/^[^a-zA-Z_\x7f-\xff]/', '/[^a-zA-Z0-9_\x7f-\xff]/');
+
     /**
      * SOAP_WSDL constructor.
      *
-     * @param string  $wsdl_uri     URL to WSDL file.
-     * @param array   $proxy        Contains options for HTTP_Request class
-     *                              @see HTTP_Request.
-     * @param boolean $cacheUse     Use WSDL caching. Defaults to false.
-     * @param integer $cacheMaxAge  Cache max lifetime (in seconds).
-     * @param boolean $docs         Parse documentation in the WSDL? Defaults
-     *                              to false.
+     * @param string $wsdl_uri          URL to WSDL file.
+     * @param array $proxy              Options for HTTP_Request class
+     *                                  @see HTTP_Request.
+     * @param boolean|string $cacheUse  Use WSDL caching? The cache directory
+     *                                  if a string.
+     * @param integer $cacheMaxAge      Cache maximum lifetime (in seconds).
+     * @param boolean $docs             Parse documentation in the WSDL?
      *
      * @access public
      */
     function SOAP_WSDL($wsdl_uri    = false,
                        $proxy       = array(),
-                       $cacheUse    = WSDL_CACHE_USE,
+                       $cacheUse    = false,
                        $cacheMaxAge = WSDL_CACHE_MAX_AGE,
                        $docs        = false)
     {
         parent::SOAP_Base('WSDL');
         $this->uri         = $wsdl_uri;
         $this->proxy       = $proxy;
-        $this->cacheUse    = $cacheUse;
+        $this->cacheUse    = !empty($cacheUse);
         $this->cacheMaxAge = $cacheMaxAge;
         $this->docs        = $docs;
+        if (is_string($cacheUse)) {
+            $this->cacheDir = $cacheUse;
+        }
 
         if ($wsdl_uri) {
             if (!PEAR::isError($this->parseURL($wsdl_uri))) {
@@ -129,21 +185,26 @@ class SOAP_WSDL extends SOAP_Base
         }
     }
 
+    /**
+     * @deprecated  Use setService().
+     */
     function set_service($service)
+    {
+        $this->setService($service);
+    }
+
+    /**
+     * Sets the service currently to be used.
+     *
+     * @param string $service  An (existing) service name.
+     */
+    function setService($service)
     {
         if (array_key_exists($service, $this->services)) {
             $this->service = $service;
         }
     }
 
-    /**
-     * @deprecated use parseURL instead
-     */
-    function parse($wsdl_uri, $proxy = array())
-    {
-        $this->parseURL($wsdl_uri, $proxy);
-    }
-
     /**
      * Fills the WSDL array tree with data from a WSDL file.
      *
@@ -185,8 +246,8 @@ class SOAP_WSDL extends SOAP_Base
 
     function getEndpoint($portName)
     {
-        if ($this->__isfault()) {
-            return $this->__getfault();
+        if ($this->_isfault()) {
+            return $this->_getfault();
         }
 
         return (isset($this->services[$this->service]['ports'][$portName]['address']['location']))
@@ -215,8 +276,8 @@ class SOAP_WSDL extends SOAP_Base
      */
     function getPortName($operation, $service = null)
     {
-        if ($this->__isfault()) {
-            return $this->__getfault();
+        if ($this->_isfault()) {
+            return $this->_getfault();
         }
 
         if (!$service) {
@@ -241,8 +302,8 @@ class SOAP_WSDL extends SOAP_Base
 
     function getOperationData($portName, $operation)
     {
-        if ($this->__isfault()) {
-            return $this->__getfault();
+        if ($this->_isfault()) {
+            return $this->_getfault();
         }
 
         if (!isset($this->services[$this->service]['ports'][$portName]['binding']) ||
@@ -314,8 +375,8 @@ class SOAP_WSDL extends SOAP_Base
 
     function matchMethod(&$operation)
     {
-        if ($this->__isfault()) {
-            return $this->__getfault();
+        if ($this->_isfault()) {
+            return $this->_getfault();
         }
 
         // Overloading lowercases function names :(
@@ -380,8 +441,8 @@ class SOAP_WSDL extends SOAP_Base
 
     function getSoapAction($portName, $operation)
     {
-        if ($this->__isfault()) {
-            return $this->__getfault();
+        if ($this->_isfault()) {
+            return $this->_getfault();
         }
 
         if (!empty($this->bindings[$this->services[$this->service]['ports'][$portName]['binding']]['operations'][$operation]['soapAction'])) {
@@ -393,8 +454,8 @@ class SOAP_WSDL extends SOAP_Base
 
     function getNamespace($portName, $operation)
     {
-        if ($this->__isfault()) {
-            return $this->__getfault();
+        if ($this->_isfault()) {
+            return $this->_getfault();
         }
 
         if (!empty($this->bindings[$this->services[$this->service]['ports'][$portName]['binding']]['operations'][$operation]['input']['namespace'])) {
@@ -520,8 +581,8 @@ class SOAP_WSDL extends SOAP_Base
      */
     function generateProxyCode($port = '', $classname = '')
     {
-        if ($this->__isfault()) {
-            return $this->__getfault();
+        if ($this->_isfault()) {
+            return $this->_getfault();
         }
 
         $multiport = count($this->services[$this->service]['ports']) > 1;
@@ -542,7 +603,7 @@ class SOAP_WSDL extends SOAP_Base
             } else {
                 $classname = 'WebService_' . $this->service;
             }
-            $classname = preg_replace('/[ .\-\(\)]+/', '_', $classname);
+            $classname = $this->_sanitize($classname);
         }
 
         if (!$this->_validateString($classname)) {
@@ -611,6 +672,7 @@ class SOAP_WSDL extends SOAP_Base
             foreach ($operation['input'] as $argname => $argtype) {
                 if ($argname == 'message') {
                     foreach ($this->messages[$argtype] as $_argname => $_argtype) {
+                        $_argname = $this->_sanitize($_argname);
                         if ($opstyle == 'document' && $use == 'literal' &&
                             $_argtype['name'] == 'parameters') {
                             // The type or element refered to is used for
@@ -626,6 +688,7 @@ class SOAP_WSDL extends SOAP_Base
                             }
                             if (isset($el['elements'])) {
                                 foreach ($el['elements'] as $elname => $elattrs) {
+                                    $elname = $this->_sanitize($elname);
                                     // Is the element a complex type?
                                     if (isset($this->complexTypes[$elattrs['namespace']][$elname])) {
                                         $comments .= $this->_complexTypeArg($args, $argarray, $_argtype, $_argname);
@@ -658,7 +721,7 @@ class SOAP_WSDL extends SOAP_Base
             // legal. This could potentially cause collisions, but let's try
             // to make everything callable and see how many problems that
             // causes.
-            $opname_php = preg_replace('/[ .\-\(\)]+/', '_', $opname);
+            $opname_php = $this->_sanitize($opname);
             if (!$this->_validateString($opname_php)) {
                 return null;
             }
@@ -676,7 +739,7 @@ class SOAP_WSDL extends SOAP_Base
                 "                                    'soapaction' => '$soapaction',\n" .
                 "                                    'style' => '$opstyle',\n" .
                 "                                    'use' => '$use'" .
-                ($this->trace?",\n                                    'trace' => 1" : '') . "));\n" .
+                ($this->trace ? ",\n                                    'trace' => true" : '') . "));\n" .
                 "        return \$result;\n" .
                 "    }\n";
         }
@@ -698,8 +761,8 @@ class SOAP_WSDL extends SOAP_Base
 
     function &getProxy($port = '', $name = '')
     {
-        if ($this->__isfault()) {
-            $fault =& $this->__getfault();
+        if ($this->_isfault()) {
+            $fault =& $this->_getfault();
             return $fault;
         }
 
@@ -720,7 +783,7 @@ class SOAP_WSDL extends SOAP_Base
             $classname = $name . '_' . $classname;
         }
 
-        $classname = preg_replace('/[ .\-\(\)]+/', '_', $classname);
+        $classname = $this->_sanitize($classname);
         if (!class_exists($classname)) {
             $proxy = $this->generateProxyCode($port, $classname);
             require_once 'SOAP/Client.php';
@@ -731,6 +794,24 @@ class SOAP_WSDL extends SOAP_Base
         return $proxy;
     }
 
+    /**
+     * Sanitizes a SOAP value, method or class name so that it can be used as
+     * a valid PHP identifier. Invalid characters are converted into
+     * underscores and reserved words are prefixed with an underscore.
+     *
+     * @param string $name  The identifier to sanitize.
+     *
+     * @return string  The sanitized identifier.
+     */
+    function _sanitize($name)
+    {
+        $name = preg_replace($this->_invalid, '_', $name);
+        if (in_array($name, $this->_reserved)) {
+            $name = '_' . $name;
+        }
+        return $name;
+    }
+
     function &_getComplexTypeForElement($name, $namespace)
     {
         $t = null;
@@ -758,12 +839,17 @@ class SOAP_WSDL extends SOAP_Base
 
     function getComplexTypeChildType($ns, $name, $child_ns, $child_name)
     {
-        // is the type an element?
+        // Is the type an element?
         $t = $this->_getComplexTypeForElement($name, $ns);
         if ($t) {
-            // no, get it from complex types directly
+            // No, get it from complex types directly.
             if (isset($t['elements'][$child_name]['type']))
                 return $t['elements'][$child_name]['type'];
+        } elseif (isset($this->ns[$ns]) &&
+                  isset($this->elements[$this->ns[$ns]][$name]['complex']) &&
+                  $this->elements[$this->ns[$ns]][$name]['complex']) {
+            // Type is not an element but complex.
+            return $this->elements[$this->ns[$ns]][$name]['elements'][$child_name]['type'];
         }
         return null;
     }
@@ -776,38 +862,37 @@ class SOAP_WSDL extends SOAP_Base
             // XXX TODO:
             // look up the name in the wsdl and validate the type.
             foreach ($this->complexTypes as $ns => $types) {
-                if (array_key_exists($type, $types)) {
-                    if (array_key_exists('type', $types[$type])) {
-                        list($arraytype_ns, $arraytype, $array_depth) = isset($types[$type]['arrayType'])?
-                            $this->_getDeepestArrayType($types[$type]['namespace'], $types[$type]['arrayType'])
+                if (isset($types[$type])) {
+                    if (isset($types[$type]['type'])) {
+                        list($arraytype_ns, $arraytype, $array_depth) = isset($types[$type]['arrayType'])
+                            ? $this->_getDeepestArrayType($types[$type]['namespace'], $types[$type]['arrayType'])
                             : array($this->namespaces[$types[$type]['namespace']], null, 0);
                         return array($types[$type]['type'], $arraytype, $arraytype_ns, $array_depth);
                     }
-                    if (array_key_exists('arrayType', $types[$type])) {
+                    if (isset($types[$type]['arrayType'])) {
                         list($arraytype_ns, $arraytype, $array_depth) =
                             $this->_getDeepestArrayType($types[$type]['namespace'], $types[$type]['arrayType']);
                         return array('Array', $arraytype, $arraytype_ns, $array_depth);
                     }
-                    if (array_key_exists('elements', $types[$type]) &&
-                        array_key_exists($name, $types[$type]['elements'])) {
+                    if (!empty($types[$type]['elements'][$name])) {
                         $type = $types[$type]['elements']['type'];
                         return array($type, null, $this->namespaces[$types[$type]['namespace']], null);
                     }
+                    break;
                 }
             }
         }
         if ($type && $type_namespace) {
             $arrayType = null;
             // XXX TODO:
-            // this code currently handles only one way of encoding array types in wsdl
-            // need to do a generalized function to figure out complex types
+            // this code currently handles only one way of encoding array
+            // types in wsdl need to do a generalized function to figure out
+            // complex types
             $p = $this->ns[$type_namespace];
-            if ($p &&
-                array_key_exists($p, $this->complexTypes) &&
-                array_key_exists($type, $this->complexTypes[$p])) {
+            if ($p && !empty($this->complexTypes[$p][$type])) {
                 if ($arrayType = $this->complexTypes[$p][$type]['arrayType']) {
                     $type = 'Array';
-                } elseif ($this->complexTypes[$p][$type]['order']=='sequence' &&
+                } elseif ($this->complexTypes[$p][$type]['order'] == 'sequence' &&
                           array_key_exists('elements', $this->complexTypes[$p][$type])) {
                     reset($this->complexTypes[$p][$type]['elements']);
                     // assume an array
@@ -872,45 +957,60 @@ class SOAP_WSDL extends SOAP_Base
 
 class SOAP_WSDL_Cache extends SOAP_Base
 {
-    // Cache settings
-
     /**
-     * Use WSDL cache
+     * Use WSDL cache?
      *
      * @var boolean
      */
-    var $_cacheUse = null;
+    var $_cacheUse;
 
     /**
-     * Cache max lifetime (in seconds)
+     * WSDL cache directory.
      *
-     * @var int
+     * @var string
      */
-    var $_cacheMaxAge = null;
+    var $_cacheDir;
 
     /**
-     * SOAP_WSDL_Cache constructor
+     * Cache maximum lifetime (in seconds)
      *
-     * @param  boolean use caching
-     * @param  int     cache max lifetime (in seconds)
-     * @access public
+     * @var integer
      */
-    function SOAP_WSDL_Cache($cacheUse = WSDL_CACHE_USE,
-                             $cacheMaxAge = WSDL_CACHE_MAX_AGE)
+    var $_cacheMaxAge;
+
+    /**
+     * Constructor.
+     *
+     * @param boolean $cashUse      Use caching?
+     * @param integer $cacheMaxAge  Cache maximum lifetime (in seconds)
+     */
+    function SOAP_WSDL_Cache($cacheUse = false,
+                             $cacheMaxAge = WSDL_CACHE_MAX_AGE,
+                             $cacheDir = null)
     {
         parent::SOAP_Base('WSDLCACHE');
         $this->_cacheUse = $cacheUse;
+        $this->_cacheDir = $cacheDir;
         $this->_cacheMaxAge = $cacheMaxAge;
     }
 
     /**
-     * _cacheDir
-     * return the path to the cache, if it doesn't exist, make it
+     * Returns the path to the cache and creates it, if it doesn't exist.
+     *
+     * @private
+     *
+     * @return string  The directory to use for the cache.
      */
     function _cacheDir()
     {
-        $dir = getenv("WSDLCACHE");
-        if (!$dir) $dir = " ./wsdlcache";
+        if (!empty($this->_cacheDir)) {
+            $dir = $this->_cacheDir;
+        } else {
+            $dir = getenv('WSDLCACHE');
+            if (empty($dir)) {
+                $dir = './wsdlcache';
+            }
+        }
         @mkdir($dir, 0700);
         return $dir;
     }
@@ -930,7 +1030,7 @@ class SOAP_WSDL_Cache extends SOAP_Base
         $cachename = $md5_wsdl = $file_data = '';
         if ($this->_cacheUse) {
             // Try to retrieve WSDL from cache
-            $cachename = SOAP_WSDL_Cache::_cacheDir() . '/' . md5($wsdl_fname). ' .wsdl';
+            $cachename = $this->_cacheDir() . '/' . md5($wsdl_fname). ' .wsdl';
             if (file_exists($cachename)) {
                 $wf = fopen($cachename, 'rb');
                 if ($wf) {
@@ -964,11 +1064,7 @@ class SOAP_WSDL_Cache extends SOAP_Base
                 if (!file_exists($wsdl_fname)) {
                     return $this->_raiseSoapFault("Unable to read local WSDL $wsdl_fname", $wsdl_fname);
                 }
-                if (function_exists('file_get_contents')) {
-                    $file_data = file_get_contents($wsdl_fname);
-                } else {
-                    $file_data = implode('',file($wsdl_fname));
-                }
+                $file_data = file_get_contents($wsdl_fname);
             } else {
                 $uri = explode('?', $wsdl_fname);
                 $rq =& new HTTP_Request($uri[0], $proxy_params);
@@ -1049,12 +1145,14 @@ class SOAP_WSDL_Parser extends SOAP_Base
     var $currentElement;
 
     /**
-     * constructor
+     * Constructor.
      */
     function SOAP_WSDL_Parser($uri, &$wsdl, $docs = false)
     {
         parent::SOAP_Base('WSDLPARSER');
-        $this->cache =& new SOAP_WSDL_Cache($wsdl->cacheUse, $wsdl->cacheMaxAge);
+        $this->cache =& new SOAP_WSDL_Cache($wsdl->cacheUse,
+                                            $wsdl->cacheMaxAge,
+                                            $wsdl->cacheDir);
         $this->uri = $uri;
         $this->wsdl = &$wsdl;
         $this->docs = $docs;
@@ -1338,6 +1436,7 @@ class SOAP_WSDL_Parser extends SOAP_Base
             $ns = $qname->ns ? $this->wsdl->namespaces[$qname->ns] : SCHEMA_WSDL;
             switch ($ns) {
             case SCHEMA_SOAP:
+            case SCHEMA_SOAP12:
                 // this deals with wsdl section 3 soap binding
                 switch ($qname->name) {
                 case 'binding':
@@ -1927,32 +2026,27 @@ class SOAP_WSDL_ObjectParser extends SOAP_Base
     }
 
     /**
-     * Parser - takes a single object to add to tree
-     * (non-destructive).
+     * Parser - takes a single object to add to tree (non-destructive).
      *
-     * @param  $object Reference to the object to parse
-     * @param  $service_name Name of the WSDL <service>
-    * @access private
+     * @access private
+     *
+     * @param object $object           Reference to the object to parse.
+     * @param string $schemaNamespace
+     * @param string $service_name     Name of the WSDL <service>.
      */
     function _parse(&$object, $schemaNamespace, $service_name)
     {
         // Create namespace prefix for the schema
-        // XXX not very elegant :-(
-
-        list($schPrefix, $foo) = $this->_getTypeNs('{' . $schemaNamespace.'}');
-        unset($foo);
+        list($schPrefix,) = $this->_getTypeNs('{' . $schemaNamespace . '}');
 
         // Parse all the types defined by the object in whatever
         // schema language we are using (currently __typedef arrays)
         // *** <wsdl:types> ***
-
         foreach ($object->__typedef as $typeName => $typeValue) {
             // Get/create namespace definition
-
             list($nsPrefix, $typeName) = $this->_getTypeNs($typeName);
 
             // Create type definition
-
             $this->wsdl->complexTypes[$schPrefix][$typeName] = array('name' => $typeName);
             $thisType =& $this->wsdl->complexTypes[$schPrefix][$typeName];
 
@@ -1960,19 +2054,17 @@ class SOAP_WSDL_ObjectParser extends SOAP_Base
             // flavors:
             // Array = array(array("item" => "value"))
             // Struct = array("item1" => "value1", "item2" => "value2", ...)
-
             if (is_array($typeValue)) {
                 if (is_array(current($typeValue)) && count($typeValue) == 1
                     && count(current($typeValue)) == 1) {
                     // It's an array
-
                     $thisType['type'] = 'Array';
-                    list($nsPrefix, $typeName) = $this->_getTypeNs(current(current($typeValue)));
+                    $nsType = current(current($typeValue));
+                    list($nsPrefix, $typeName) = $this->_getTypeNs($nsType);
                     $thisType['namespace'] = $nsPrefix;
                     $thisType['arrayType'] = $typeName . '[]';
                 } elseif (!is_array(current($typeValue))) {
                     // It's a struct
-
                     $thisType['type'] = 'Struct';
                     $thisType['order'] = 'all';
                     $thisType['namespace'] = $nsPrefix;
@@ -1996,12 +2088,10 @@ class SOAP_WSDL_ObjectParser extends SOAP_Base
 
         // Create an empty element array with the target namespace
         // prefix, to match the results of WSDL parsing.
-
         $this->wsdl->elements[$schPrefix] = array();
 
         // Populate tree with message information
         // *** <wsdl:message> ***
-
         foreach ($object->__dispatch_map as $operationName => $messages) {
             foreach ($messages as $messageType => $messageParts) {
                 unset($thisMessage);
@@ -2044,7 +2134,6 @@ class SOAP_WSDL_ObjectParser extends SOAP_Base
         // XXX Current implementation only supports one portType that
         // encompasses all of the operations available.
         // *** <wsdl:portType> ***
-
         if (!isset($this->wsdl->portTypes[$service_name . 'Port'])) {
             $this->wsdl->portTypes[$service_name . 'Port'] = array();
         }
@@ -2117,11 +2206,12 @@ class SOAP_WSDL_ObjectParser extends SOAP_Base
         // Populate tree with service information
         // XXX Current implementation supports one service which groups
         // all of the ports together, one port per binding
-        // XXX What about https?
         // *** <wsdl:service> ***
 
         $this->wsdl->services[$service_name . 'Service'] = array('ports' => array());
         $thisService =& $this->wsdl->services[$service_name . 'Service']['ports'];
+        $https = (isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] == 'on')) ||
+            getenv('SSL_PROTOCOL_VERSION');
 
         foreach ($this->wsdl->bindings as $bindingName => $bindingData) {
             $thisService[$bindingData['type']] = array(
@@ -2129,7 +2219,8 @@ class SOAP_WSDL_ObjectParser extends SOAP_Base
                     'binding' => $bindingName,
                     'namespace' => $this->tnsPrefix,
                     'address' => array('location' =>
-                        'http://' . $_SERVER['SERVER_NAME'] . $_SERVER['PHP_SELF'] .
+                        ($https ? 'https://' : 'http://') .
+                        $_SERVER['SERVER_NAME'] . $_SERVER['PHP_SELF'] .
                         (isset($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : '')),
                     'type' => 'soap');
         }
@@ -2164,15 +2255,15 @@ class SOAP_WSDL_ObjectParser extends SOAP_Base
      */
     function _getTypeNs($type)
     {
-        preg_match_all("'\{(.*)\}'sm", $type, $m);
-        if (isset($m[1][0]) && $m[1][0] != '') {
-            if (!array_key_exists($m[1][0], $this->wsdl->ns)) {
+        preg_match_all('/\{(.*)\}/sm', $type, $m);
+        if (!empty($m[1][0])) {
+            if (!isset($this->wsdl->ns[$m[1][0]])) {
                 $ns_pref = 'ns' . count($this->wsdl->namespaces);
                 $this->wsdl->ns[$m[1][0]] = $ns_pref;
                 $this->wsdl->namespaces[$ns_pref] = $m[1][0];
             }
             $typens = $this->wsdl->ns[$m[1][0]];
-            $type = ereg_replace($m[0][0], '', $type);
+            $type = str_replace($m[0][0], '', $type);
         } else {
             $typens = 'xsd';
         }
